//===================================CISCKER====================================
//==================================MM Registers================================
//------------------------------------------------------------------------------
//File Name:	MM_registers.v
//Purpose:		Memory Mapped generation and selection
//Parameters:	None
//------------------------------------------------------------------------------
//=====================Alfredo Gualdron - Jose P. Pinilla=======================
//=================================Semillero ADT================================
//================================UPB Bucaramanga===============================
//Website:	http://semilleroadt.upbbga.edu.co/xiscker
//E-mail:	semilleroadt@gmail.com
//------------------------------------------------------------------------------
module MM_registers(
	//Inputs
	input 	clk,
	input	tmr_clk,
	input	we,
	input 	[7:0] data,
	input	[3:0] address,
	input	[7:0] ip1,			//Input port 1
	input	[7:0] ip2,			//Input port 2
	//Outputs
	output	[7:0] op1,
	output	[7:0] op2,
	output	[7:0] int_ctrl,
	output	int_timer1,
	output	int_timer2,
	output	[7:0] out
	);

//Read-Write Registers
reg 	[7:0]registers[7:0];/*	
16'h8000	Interruption Control
16'h8001	Timer Control
16'h8002	Timer1 Period [15:8]
16'h8003	Timer1 Period [7:0]
16'h8004	Timer2 Period [15:8]
16'h8005	Timer2 Period [7:0]
16'h8006	Output Port 1
16'h8007	Output Port 2*/	

/*Read-Only Registers
16'h8008	Input Port 1
16'h8009	Input Port 2
*/
wire 	[15:0] tmr1_counter;		/*Timer1 value
16'h800A	Timer1	[15:8]
16'h800B	Timer1	[7:0]				*/
wire 	[15:0] tmr2_counter;		/*Timer2 value
16'h800C	Timer2	[15:8]
16'h800D	Timer2	[7:0]				
*/

always @ (posedge clk)
	if (we)
		registers[address] <= data;

wire [7:0]reg_bus[13:0];
genvar i;
generate
	for (i=0; i <= 13; i=i+1) 
		begin: assignments
			if (i==8) 
				assign reg_bus[8] = ip1;
			else
			if (i==9) 
				assign reg_bus[9] = ip2;
			else
			if (i==10) 
				assign reg_bus[10] = tmr1_counter[15:8];
			else
			if (i==11) 
				assign reg_bus[11] = tmr1_counter[7:0];
			else
			if (i==12) 
				assign reg_bus[12] = tmr2_counter[15:8];
			else
			if (i==13) 
				assign reg_bus[13] = tmr2_counter[7:0];
			else
				assign reg_bus[i] = registers[i];
		end
endgenerate

assign int_ctrl = reg_bus[0];
assign op1 		= reg_bus[6];
assign op2 		= reg_bus[7];

assign out = reg_bus[address];

//Timers Modules

//Timer Control Register
wire [1:0]tmr1_prescaler	= reg_bus[1][7:6];
wire [1:0]tmr2_prescaler	= reg_bus[1][5:4];
wire tmr1_enable			= reg_bus[1][3];
wire tmr2_enable			= reg_bus[1][2];
wire tmr1_reset				= reg_bus[1][1];
wire tmr2_reset				= reg_bus[1][0];

wire [15:0] tmr1_period = {reg_bus[2],reg_bus[3]};
wire [15:0] tmr2_period = {reg_bus[4],reg_bus[5]};

timer	timer_1 (	.clk(tmr_clk), 				//input clk, 
					.prescaler(tmr1_prescaler),      //input [1:0] prescaler, 
					.period(tmr1_period),            //input[15:0] period, 
					.enable(tmr1_enable),            //input enable,
					.reset_tmr(tmr1_reset),      //input reset_tmr,
					.int_timer(int_timer1),      //output reg int_timer,
					.counter(tmr1_counter) 			//output reg [15:0] counter=0 
				);
		
timer	timer_2	(	.clk(tmr_clk), 				//input clk, 
					.prescaler(tmr2_prescaler),      //input [1:0] prescaler, 
					.period(tmr2_period),            //input[15:0] period, 
					.enable(tmr2_enable),            //input enable,
					.reset_tmr(tmr2_reset),      //input reset_tmr,
					.int_timer(int_timer2),      //output reg int_timer,
					.counter(tmr2_counter) 			//output reg [15:0] counter=0 
				);

endmodule 