//===================================CISCKER====================================
//================================Next Address==================================
//------------------------------------------------------------------------------
//File Name:	next_address.v
//Purpose:		Returns the Internal Address Bus (IAB) that will read from memory
//Parameters:	None
//------------------------------------------------------------------------------
//=====================Alfredo Gualdron - Jose P. Pinilla=======================
//=================================Semillero ADT================================
//================================UPB Bucaramanga===============================
//Website:	http://semilleroadt.upbbga.edu.co/xiscker
//E-mail:	semilleroadt@gmail.com
//------------------------------------------------------------------------------
module next_address (
			//control
			input adder_cin, 
			input [1:0] adder_src_a,
			input [1:0] adder_src_b,
			input pass_adder,
			//in
			input [15:0] PC,
			input [15:0] SP,
			input [15:0] IX,
			input [15:0] MAH,
			//out
			output [15:0] adder,
			output [15:0] IAB //Internal Address Bus
			);

reg [15:0] src_a;
reg [15:0] src_b;
parameter zero = 16'h0000;


always @(*)
begin
	case (adder_src_a)
		2'b00: src_a = PC;
		2'b01: src_a = SP;
		2'b10: src_a = IX;
		2'b11: src_a = MAH;
	endcase
end

always @(*)
begin
	case(adder_src_b)
		2'b00: src_b =  zero;
		2'b01: src_b =	16'hffff;
		2'b10: src_b =  16'h0001;
		2'b11: src_b =  MAH;
	endcase
end
	
assign adder = src_a + src_b + adder_cin;	

assign IAB = (pass_adder) ? adder : src_a;


endmodule
